# -*- coding: utf-8 -*-
# @File : windows.py
# @Author : mocobk
# @Email : mailmzb@qq.com
# @Time : 2020/11/29 3:13 下午
import os
from ctypes import windll, create_unicode_buffer, c_void_p, c_uint, c_wchar_p
from typing import List
from plyer import notification

def get_clipboard_file_paths() -> List[str]:
    """
    windows 平台获取剪贴板复制的文件路径列表
    """
    kernel32 = windll.kernel32
    global_lock = kernel32.GlobalLock
    global_lock.argtypes = [c_void_p]
    global_lock.restype = c_void_p
    global_unlock = kernel32.GlobalUnlock
    global_unlock.argtypes = [c_void_p]

    user32 = windll.user32
    get_clipboard_data = user32.GetClipboardData
    get_clipboard_data.restype = c_void_p

    drag_query_file = windll.shell32.DragQueryFileW
    drag_query_file.argtypes = [c_void_p, c_uint, c_wchar_p, c_uint]

    cf_hdrop = 15
    paths_list = []

    if user32.OpenClipboard(None):
        h_global = user32.GetClipboardData(cf_hdrop)
        if h_global:
            h_drop = global_lock(h_global)
            if h_drop:
                count = drag_query_file(h_drop, 0xFFFFFFFF, None, 0)
                for i in range(count):
                    length = drag_query_file(h_drop, i, None, 0)
                    buffer = create_unicode_buffer(length)
                    drag_query_file(h_drop, i, buffer, length + 1)
                    paths_list.append(buffer.value)

                global_unlock(h_global)

        user32.CloseClipboard()

    return paths_list

def notify(title='UpImg', message=''):
    """发送系统通知"""
    icon_path = os.path.join(os.path.dirname(__file__), 'notification.ico')
    notification.notify(title, message, app_icon=icon_path,  timeout=0)
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : mocobk
# @Email  : mailmzb@qq.com
# @Time   : 2020/11/30 16:24
import argparse
import os
from traceback import print_exc
from urllib.parse import urljoin

from pyperclip import copy as copy2clipboard

from upimg import utils
from upimg.upload import UpImage


def parse_args():
    parser = argparse.ArgumentParser(description='Upload image from clipboard and return Markdown link.')
    parser.add_argument('-c', '--config', action="store_true", help='file upload config')
    return parser.parse_args()


def main():
    args = parse_args()
    if args.config:
        utils.set_config()
        print('Config success! Now you can execute the command "upimg"')

    else:
        if not os.path.exists(utils.CONFIG_PATH):
            message = 'Config file is not found, Please config it first.\nusage: upimg --config'
            utils.send_notify(*message.split('\n'))
            raise Exception(message)
        config = utils.get_config()
        try:
            # utils.alt_tab()
            # listener = utils.ListenerOperation()
            up = UpImage(config.service, config.username, config.password, config.upload_path)
            utils.send_notify(title='UpImg', message='uploading ...')
            markdown_links = [f'![]({urljoin(config.url_base, path)})' for path in up.upload()]
            # copy2clipboard('\n'.join(markdown_links))
            # if not listener.HAS_OPERATED:
            #     utils.clipboard_paste()
            files_count = len(markdown_links)
            if files_count > 0:
                utils.send_notify(title=f'{files_count} upload success', message='You can paste it with Ctrl + V ')
            else:
                utils.send_notify(title=f'{files_count} upload success', message='Maybe you should copy it first')
            copy2clipboard('\n'.join(markdown_links))

        # except KeyboardInterrupt:
        #     utils.send_notify(title=f'No Files', message='Please copy it first')
        except Exception:
            with open('./upimg-error.log', 'w') as fp:
                print_exc(file=fp)
            utils.send_notify('Error!!!', message=f'{os.path.abspath("./upimg-error.log")}')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# @File : upload.py
# @Author : mocobk
# @Email : mailmzb@qq.com
# @Time : 2020/11/25 10:54 下午

from datetime import datetime
from typing import List

import upyun

from upimg.utils import ClipbordFile


class UpImage:
    def __init__(self, service, username, password, upload_path='/'):
        self.upyun = upyun.UpYun(service, username, password)
        self.upload_path = upload_path

    def upload(self) -> List[str]:
        """
        return {'content-length': '1916', 'content-type': 'image/jpeg', 'etag': '"ad38e734e48cd6134e02bc60f91c55b5"',
        'file-type': 'JPEG', 'width': '132', 'height': '23', 'frames': '1'}
        """
        path_list = []
        clipbord_file = ClipbordFile()
        for file, ext in clipbord_file.file_objects:
            full_path = self.upload_path + datetime.now().strftime('%Y%m%d%H%M%S%f') + ext
            self.upyun.put(full_path, file)
            path_list.append(full_path)
        return path_list


if __name__ == '__main__':
    up = UpImage('mocobk', 'mocobk', 'mo123456')
    print(up.upload())



# -*- coding: utf-8 -*-
# @File : mac.py
# @Author : mocobk
# @Email : mailmzb@qq.com
# @Time : 2020/11/29 3:14 下午
from typing import List

from AppKit import NSPasteboard, NSFilenamesPboardType
from notifypy import Notify


def get_clipboard_file_paths() -> List[str]:
    """
    获取剪贴板复制的文件路径列表
    """
    pb = NSPasteboard.generalPasteboard()
    if NSFilenamesPboardType in pb.types():
        paths = pb.propertyListForType_(NSFilenamesPboardType)
        return list(paths)
    else:
        return []

def notify(title='UpImg', message='') -> bool:
    """发送系统通知"""
    notify = Notify()
    notify.title = title
    notify.message = message
    return notify.send(block=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author : mocobk
# @Email  : mailmzb@qq.com
# @Time   : 2020/11/26 19:14
import os
import pickle
import platform
import time
from collections import namedtuple
from io import BytesIO

from PIL import Image, ImageGrab
from pynput import keyboard, mouse
from pynput.keyboard import Controller, Key

SYSTEM = platform.system()
if SYSTEM == 'Windows':
    from upimg.sys_platform.windows import get_clipboard_file_paths, notify

    CTRL_KEY = Key.ctrl

if SYSTEM == 'Darwin':
    from upimg.sys_platform.mac import get_clipboard_file_paths, notify

    CTRL_KEY = Key.cmd


class ClipbordFile:
    @property
    def clipboard_file_paths(self):
        return get_clipboard_file_paths()

    @property
    def file_objects(self):
        """
        获取图片优先级： 剪贴板图片 > 已复制到剪贴板的文件列表 > 已选择的文件
        """

        image = ImageGrab.grabclipboard()

        if isinstance(image, Image.Image):
            with BytesIO() as image_bytes_io:
                image.save(image_bytes_io, format='PNG')
                image_bytes_io.seek(0)
                yield image_bytes_io, '.png'
            return

        file_paths = self.clipboard_file_paths
        if file_paths:
            for path in file_paths:
                with open(path, 'rb') as fp:
                    ext = os.path.splitext(fp.name)[1].lower()
                    yield fp, ext


def send_notify(title='UpImg', message=''):
    return notify(title=title, message=message)


class ListenerOperation:
    """
    监听是否键盘按键和鼠标点击操作
    """
    HAS_OPERATED = False

    def __init__(self):
        keyboard_listener = keyboard.Listener(on_release=self.listener_operation)
        mouse_listener = mouse.Listener(on_click=self.listener_operation)
        keyboard_listener.start()
        mouse_listener.start()

    def listener_operation(self, value, *args):
        if self.HAS_OPERATED:
            return False
        if value:
            self.HAS_OPERATED = True
            return False


KEYBOARD = Controller()


def clipboard_copy():
    """模拟 ctrl + c 复制已选择的文件到剪贴板"""
    with KEYBOARD.pressed(CTRL_KEY):
        KEYBOARD.press('c')
        KEYBOARD.release('c')


def clipboard_paste():
    """模拟 ctrl + v 粘贴剪贴板内容"""
    with KEYBOARD.pressed(CTRL_KEY):
        KEYBOARD.press('v')
        KEYBOARD.release('v')


def alt_tab():
    """模拟 alt + tab 切换回原窗口"""
    with KEYBOARD.pressed(Key.alt):
        KEYBOARD.press(Key.tab)
        KEYBOARD.release(Key.tab)


CONFIG_PATH = os.path.join(os.path.dirname(__file__), 'config.pkl')
CONFIG_FIELDS = ['service', 'username', 'password', 'upload_path', 'url_base']
CONFIG_DEFAULTS = ['', '', '', '/', 'https://test.upimg.com']
CONFIG = namedtuple('CONFIG', CONFIG_FIELDS)


def set_config():
    args = {}
    for field, default in zip(CONFIG_FIELDS, CONFIG_DEFAULTS):
        value = input('{}{}: '.format(field.replace('_', ' '), f'[{default}]' if default else ''))
        args[field] = value or default

    with open(CONFIG_PATH, 'wb') as fp:
        pickle.dump(CONFIG(**args), fp)


def get_config() -> CONFIG:
    with open(CONFIG_PATH, 'rb') as fp:
        return pickle.load(fp)


if __name__ == '__main__':
    pass




import io
from setuptools import setup, find_packages

with io.open("README.md", encoding="utf-8") as f:
    README = f.read()

setup(
    name="upimg",
    version="0.1.4",
    description="A CLI tool to genarate Markdown links from clipboard",
    url="https://github.com/mocobk/upimg",
    long_description=README,
    long_description_content_type="text/markdown",
    author="mocobk",
    author_email="mailmzb@163.com",
    license="MIT",
    install_requires=["notify-py; sys_platform == 'darwin'",
                      "pillow",
                      "plyer; sys_platform == 'win32'",
                      "pynput",
                      "pyobjc; sys_platform == 'darwin'",
                      "pyperclip",
                      "upyun"],
    packages=find_packages('.'),
    package_data={'upimg.sys_platform': ['notification.ico']},
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    entry_points={"console_scripts": ["upimg = upimg.cli:main"]},
)


